#ifndef __REFSTRIZER__
#define __REFSTRIZER__

#include <stddef.h>

/*
* refstrs are 'unique' strings.
* there is guaranteed to be only one copy of a given refstr string at a time.
* this means that refstrs can be compared for equality by comparing their addresses.
* ex:
* 	if (A_rstr_p == B_rstr_p) ...
*/
typedef const char refstr;

/* refstr_nget
* string - a c-string to get a refstr for
* s_len  - the length of the c-string, as strlen would return it
* force  -  0 : dont make a refstr if there isn't one already
*          !0 : make a refstr if there isn't one already.
*
* NULL is returned if no refstr is found and force is not set,
* otherwise, the returned refstr is ref'd once before return.
*/
refstr *		refstr_nget	(const char *string, size_t s_len, int force);

/* a wrapper arround refstr_nget that does a strlen first */
refstr *		refstr_get	(const char *string, int force);

/* the length of the refstr (cheaper than strlen) */
size_t			refstr_len	(refstr * rstr_p);

/* bump the refcount of the refstr 
* a cheap way of 'copying' refstrs is:
* 	copy_rstr_p = refstr_ref(rstr_p);
*/
refstr * 		refstr_ref	(refstr * rstr_p);

/* drop the refcount of the refstr */
void			refstr_unref	(refstr * rstr_p);

#endif/*__REFSTRIZER__*/
